{**********************************************************************}
{* NAND gates D-type latch (variant) unit for LogicGates5 application *}
{**********************************************************************}

unit d1b;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, common;

type
  {*******}
  { TfD1b }
  {*******}
  TfD1b = class(TForm)
    stTitle: TStaticText;
    imGate1, imGate2, imGate3, imGate4: TImage;
    shSwitch1a, shSwitch1b, shSwitch3a, shSwitch3b: TShape;
    imSwitch1, imSwitch3: TImage;
    shLedIn1, shLedIn2, shLedIn1b, shLedIn2b, shLedInEN, shLedOut1, shLedOut2: TShape;
    Shape1, Shape2, Shape3, Shape4, Shape5, Shape6, Shape7, Shape8, Shape9, Shape10: TShape;
    Shape11, Shape12, Shape13, Shape14, Shape15, Shape16, Shape17, Shape18, Shape19, Shape20: TShape;
    Shape21, Shape22, Shape23, Shape24, Shape25, Shape26, Shape27, Shape28, Shape29, Shape31: TShape;
    Shape32, Shape37, Shape38, Shape39, Shape40, Shape41, Shape42, Shape44, Shape45, Shape48: TShape;
    Shape49, Shape50, Shape51, Shape52, Shape53, Shape54, Shape55, Shape56, Shape57: TShape;
    StaticText1, StaticText2, StaticText3, StaticText4, StaticText5, StaticText6, StaticText7: TStaticText;
    StaticText8, StaticText9, StaticText10, StaticText11, StaticText12, StaticText13: TStaticText;
    Label1, Label2, Label3, Label4, Label5, Label7, Label8, Label9, Label10: TLabel;
    Label12, Label13, Label14, Label15, Label16, Label18, Label19, Label20: TLabel;
    Label21, Label22, Label24, Label25, Label26, Label27, Label28, Label30: TLabel;
    Label31, Label32, Label33, Label34, Label36, Label37, Label38, Label39, Label40: TLabel;
    Label41, Label47, Label48, Label49, Label50, Label51, Label54, Label55: TLabel;
    Memo1: TMemo;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitch1Click(Sender: TObject);
    procedure imSwitch3Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    E, D, Q, Qbar: Boolean;
    SwitchSetReset, SwitchEnable: TSwitch;
    LedE, LedD, LedDbar, LedSbar, LedRbar, LedQ, LedQbar: TLed;
  end;

var
  fD1b: TfD1b;

implementation

{$R *.lfm}

{*******}
{ TfD1b }
{*******}

{ Application start: Init the LED and Switch records }

procedure TfD1b.FormCreate(Sender: TObject);

begin
  LedE.LedShape := shLedInEN;                                                  // ENABLE input LED
  LedD.LedShape := shLedIn1; LedDbar.LedShape := shLedIn2;                     // D and Dbar input LEDs
  LedSbar.LedShape := shLedIn1b; LedRbar.LedShape := shLedIn2b;                // Sbar and Rbar input LEDs
  LedQ.LedShape := shLedOut1;  LedQbar.LedShape := shLedOut2;                  // Q and Qbar output LEDs
  with SwitchSetReset do begin                                                 // SET/RESET (D) switch
    SwitchShapeA := shSwitch1a; SwitchShapeB := shSwitch1b;
    SwitchShapeAOffPos := 84;
  end;
  with SwitchEnable do begin                                                   // ENABLE switch
    SwitchShapeA := shSwitch3a; SwitchShapeB := shSwitch3b;
    SwitchShapeAOffPos := 219;
  end;
end;

{ Window show-up: Init latch circuit (initial state in table) }

procedure TfD1b.FormActivate(Sender: TObject);

begin
  SwitchOff(SwitchSetReset); SwitchOn(SwitchEnable);
  LedOn(LedE);
  LedOff(LedD); LedOn(LedDbar);
  LedOn(LedSbar); LedOff(LedRbar);
  LedOff(LedQ); LedOn(LedQbar);
  E := True;                                                                   // logic value of E input (as Boolean)
  D := False;                                                                  // logic value of D input (as Boolean)
  Q := False; Qbar := True;                                                    // logic value of Q and Qbar outputs (as Boolean)
end;

{ SET/RESET (D) switch pushed: Set corr. inputs as applicable and update simulation accordingly }

procedure TfD1b.imSwitch1Click(Sender: TObject);

// As the logic level of the input of NAND 4 equal to the output of NAND 3 is always the inverse of the level of D (thus Dbar),
// the corresponding LED is turned on/off here. A more realistic way would be to do it in the NAND D-type latch variant simulation routine

var
  SwitchAction: string;

begin
  if SwitchSetReset.SwitchOn then begin
    // Switch D was on
    SwitchOff(SwitchSetReset);
    LedOff(LedD); LedOn(LedDbar);                                              // if LED D is off, Dbar is on
    D := False;                                                                // D = logic 0
    SwitchAction := 'reset';                                                   // this corresponds to a RESET of the SR latch
  end
  else begin
    // Switch D was off
    SwitchOn(SwitchSetReset);
    LedOn(LedD); LedOff(LedDbar);                                              // if LED D is on, Dbar is off
    D := True;                                                                 // D = logic 1
    SwitchAction := 'set';                                                     // this corresponds to a SET of the SR latch
  end;
  Latch_D_nand2(SwitchAction, E, D, Q, Qbar, LedSbar, LedRbar, LedQ, LedQbar); // NAND D-type latch (variant) simulation (incl. turning on/off Sbar and Rbar LED)
end;

{ ENABLE switch pushed: Set corr. input as applicable and update simulation accordingly }

procedure TfD1b.imSwitch3Click(Sender: TObject);

begin
  if SwitchEnable.SwitchOn then begin
    // Switch EN was on
    SwitchOff(SwitchEnable);
    LedOff(LedE);
    E := False;                                                                // E = logic 0
  end
  else begin
    // Switch EN was off
    SwitchOn(SwitchEnable);
    LedOn(LedE);
    E := True;                                                                 // E = logic 1
  end;
  Latch_D_nand2('enable', E, D, Q, Qbar, LedSbar, LedRbar, LedQ, LedQbar);     // NAND D-type latch (variant) simulation (incl. turning on/off Sbar and Rbar LED)
end;

{ Button "Close": Close simulation window }

procedure TfD1b.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

