{***********************************************************}
{* NOR gates D-type latch unit for LogicGates5 application *}
{***********************************************************}

unit d2;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, common;

type
  {******}
  { TfD2 }
  {******}
  TfD2 = class(TForm)
    stTitle: TStaticText;
    imGate1, imGate2, imGate3, imGate4, imGate5: TImage;
    shSwitch1a, shSwitch1b, shSwitch3a, shSwitch3b: TShape;
    imSwitch1, imSwitch3: TImage;
    shLedIn1, shLedIn2, shLedIn1b, shLedIn2b, shLedInEN, shLedOut1, shLedOut2: TShape;
    Shape1, Shape2, Shape3, Shape4, Shape5, Shape6, Shape7, Shape8, Shape9, Shape10: TShape;
    Shape11, Shape12, Shape13, Shape14, Shape15, Shape16, Shape17, Shape18, Shape19, Shape20: TShape;
    Shape21, Shape24, Shape25, Shape26, Shape29, Shape30, Shape31, Shape32, Shape34, Shape36: TShape;
    Shape37, Shape38, Shape40, Shape41, Shape42, Shape44, Shape45, Shape48, Shape49, Shape50: TShape;
    Shape51, Shape52, Shape53, Shape54, Shape55, Shape33, Shape39: TShape;
    StaticText1, StaticText2, StaticText4, StaticText5, StaticText6, StaticText7, StaticText8: TStaticText;
    StaticText9, StaticText10, StaticText11, StaticText12, StaticText13, StaticText14: TStaticText;
    Label1, Label2, Label3, Label4, Label5, Label6, Label7, Label8, Label9, Label10: TLabel;
    Label12, Label13, Label14, Label15, Label16, Label18, Label19, Label20: TLabel;
    Label21, Label22, Label24, Label25, Label26, Label27, Label28, Label30: TLabel;
    Label31, Label32, Label33, Label34, Label36, Label37, Label38, Label39, Label40: TLabel;
    Label41, Label47, Label48, Label49, Label50, Label51, Label54, Label55: TLabel;
    Memo1: TMemo;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitch1Click(Sender: TObject);
    procedure imSwitch3Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    E, D, Q, Qbar: Boolean;
    SwitchSetReset, SwitchEnable: TSwitch;
    LedE, LedD, LedDbar, LEDS, LEDR, LedQ, LedQbar: TLed;
  end;

var
  fD2: TfD2;

implementation

{$R *.lfm}

{******}
{ TfD2 }
{******}

{ Application start: Init the LED and Switch records }

procedure TfD2.FormCreate(Sender: TObject);

begin
  LedE.LedShape := shLedInEN;                                                  // ENABLE input LED
  LedD.LedShape := shLedIn2; LedDbar.LedShape := shLedIn1;                     // D and Dbar input LEDs
  LEDS.LedShape := shLedIn2b; LEDR.LedShape := shLedIn1b;                      // S and R input LEDs
  LedQ.LedShape := shLedOut1;  LedQbar.LedShape := shLedOut2;                  // Q and Qbar output LEDs
  with SwitchSetReset do begin                                                 // SET/RESET (D) switch
    SwitchShapeA := shSwitch1a; SwitchShapeB := shSwitch1b;
    SwitchShapeAOffPos := 354;
  end;
  with SwitchEnable do begin                                                   // ENABLE switch
    SwitchShapeA := shSwitch3a; SwitchShapeB := shSwitch3b;
    SwitchShapeAOffPos := 219;
  end;
end;

{ Window show-up: Init latch circuit (initial state in table) }

procedure TfD2.FormActivate(Sender: TObject);

begin
  SwitchOff(SwitchSetReset); SwitchOn(SwitchEnable);                           // switch E on, switch D off
  LedOn(LedE);                                                                 // LED E on
  LedOff(LedD); LedOn(LedDbar);                                                // LED D off, Dbar on
  LedOff(LEDS); LedOn(LEDR);                                                   // LED S off, R on
  LedOff(LedQ); LedOn(LedQbar);                                                // LED Q off, Qbar on
  E := True;                                                                   // logic value of E input (as Boolean)
  D := False;                                                                  // logic value of D input (as Boolean)
  Q := False; Qbar := True;                                                    // logic value of Q and Qbar outputs (as Boolean)
end;

{ SET/RESET (D) switch pushed: Set corr. inputs as applicable and update simulation accordingly }

procedure TfD2.imSwitch1Click(Sender: TObject);

var
  SwitchAction: string;

begin
  if SwitchSetReset.SwitchOn then begin
    // Switch D was on
    SwitchOff(SwitchSetReset);
    LedOff(LedD); LedOn(LedDbar);                                              // if LED D is off, Dbar is on
    D := False;                                                                // D = logic 0
    SwitchAction := 'reset';                                                   // this corresponds to a RESET of the RS latch
  end
  else begin
    // Switch D was off
    SwitchOn(SwitchSetReset);
    LedOn(LedD); LedOff(LedDbar);                                              // if LED D is on, Dbar is off
    D := True;                                                                 // D = logic 0
    SwitchAction := 'set';                                                     // this corresponds to a SET of the RS latch
  end;
  Latch_D_nor(SwitchAction, E, D, Q, Qbar, LEDR, LEDS, LedQ, LedQbar);         // NOR D-type latch simulation (incl. turning on/off S and R LED)
end;

{ ENABLE switch pushed: Set corr. input as applicable and update simulation accordingly }

procedure TfD2.imSwitch3Click(Sender: TObject);

begin
  if SwitchEnable.SwitchOn then begin
    // Switch EN was on
    SwitchOff(SwitchEnable);
    LedOff(LedE);
    E := False;                                                                // E = logic 0
  end
  else begin
    // Switch EN was off
    SwitchOn(SwitchEnable);
    LedOn(LedE);
    E := True;                                                                 // E = logic 1
  end;
  Latch_D_nor('enable', E, D, Q, Qbar, LEDR, LEDS, LedQ, LedQbar);             // NOR D-type latch simulation (incl. turning on/off S and R LED)
end;

{ Button "Close": Close simulation window }

procedure TfD2.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

