{*****************************************}
{* Main unit for LogicGates5 application *}
{*****************************************}

unit logic5;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, Buttons, Menus, sr, sr2, rs, sre, rse, d1, d1b, d2;

type
  {**********}
  { TfLogic5 }
  {**********}
  TfLogic5 = class(TForm)
    mMenu: TMainMenu;
    mFile, mFileExit, mHelp, mHelpAbout: TMenuItem;
    StaticText1: TStaticText;
    Label1: TLabel;
    rbCircuit0, rbCircuit1, rbCircuit2, rbCircuit3: TRadioButton;
    rbCircuit4, rbCircuit6, rbCircuit7, rbCircuit8: TRadioButton;
    Shape1: TShape;
    stDetails: TStaticText;
    btShow: TButton;
    procedure FormCreate(Sender: TObject);
    procedure mFileExitClick(Sender: TObject);
    procedure mHelpAboutClick(Sender: TObject);
    procedure btShowClick(Sender: TObject);
    procedure rbCircuit0Change(Sender: TObject);
    procedure rbCircuit1Change(Sender: TObject);
    procedure rbCircuit2Change(Sender: TObject);
    procedure rbCircuit3Change(Sender: TObject);
    procedure rbCircuit4Change(Sender: TObject);
    procedure rbCircuit6Change(Sender: TObject);
    procedure rbCircuit7Change(Sender: TObject);
    procedure rbCircuit8Change(Sender: TObject);
  private
    iCircuit: Integer;
  end;

const
  Details_SRLatch = 'SR (RS) latches are bi-stable devices. A pulse on one of the inputs takes the output on a particular logical state. The output will then remain in this state until a similar pulse is applied to the other input. The SR latch can be considered as a 1-bit memory, since it stores the input pulse even after it has passed.';
  Details_DLatch  = 'D-type latches are constructed from gated SR latches with an inverter added between the S and the R inputs to allow for a single input. But, their big advantage consists in overcoming the major drawbacks of the SR latch, i.e. its indeterminate output and non-allowed logic states.';
  Details_Output1 = '                                                           _';
  Details_Output2 = 'The device has two outputs Q and its inverse or complement Q.';
  Details_Output1b = '                                                                           _';
  Details_Output2b = 'The device has two (corresponding) outputs Q and its inverse or complement Q.';
  Details_Circuits: array[0..8, 0..2] of AnsiString = (
    ( 'The active low SR NAND gates latch is made of a pair of cross-coupled 2-input NAND gates, so that there is feedback from each output to the other NAND gate inputs.',
      '                                                      _                              _',
      'The device has two active low inputs, one called Set (S) and the other called Reset (R).'
    ),
    ( 'The active high RS NOR gates latch is made of a pair of cross-coupled 2-input NOR gates, so that there is feedback from each output to the other NOR gate inputs.',
      '',
      'The device has two active high inputs, one called Set (S) and the other called Reset (R).'
    ),
    ( 'The active high SR NAND gates latch is made of a pair of cross-coupled 2-input NAND gates, so that there is feedback from each output to the other NAND gate inputs. Two extra NAND gates, connected as inverters to the S and R inputs, allow to control the latch with high level signals.',
      '',
      'The device has two active high inputs, one called Set (S) and the other called Reset (R).'
    ),
    ( 'The gated NAND gates SR latch is made of an SR NAND latch, controlled by an extra input signal. The latch only reacts (on active high signals) when Enable is high; otherwise the outputs remain unchanged.',
      '',
      'The device has three active high inputs, the Set (S) and Reset (R) inputs as for a basic latch, plus an Enable input EN.'
    ),
    ( 'The gated NOR gates RS latch is made of an RS NOR latch, controlled by an extra input signal. The latch only reacts (on active high signals) when Enable is high; otherwise the outputs remain unchanged.',
      '',
      'The device has three active high inputs, the Set (S) and Reset (R) inputs as for a basic latch, plus an Enable input EN.'
    ),
    ( '',
      '',
      ''
    ),
    ( 'The NAND gates D-type latch is made of a gated SR NAND latch, with an inverter added between the S and the R inputs. When Enable is low, the outputs remain unchanged, when it is high, then whichever logic state is at D will appear at output Q.',
      '',
      'The device has two active high inputs, the Data (D) input (instead of S and R for the SR latch) and the Enable input EN.'
    ),
    ( 'The NOR gates D-type latch is made of a gated RS NOR latch, with an inverter added between the S and the R inputs. When Enable is low, the outputs remain unchanged, when it is high, then whichever logic state is at D will appear at output Q.',
      '',
      'The device has two active high inputs, the Data (D) input (instead of R and S for the SR latch) and the Enable input EN.'
    ),
    ( 'This is a variant of the classical NAND gates D-type latch (you may also say a simplification, as no inverter is needed): the inversion of the logical level at D (input signal for the first NAND), to be used as input for the second NAND is done by connecting the output of the first NAND to the input of the second one.',
      '',
      'The device has two active high inputs, the Data (D) input (instead of S and R for the SR latch) and the Enable input EN.'
    )
  );

var
  fLogic5: TfLogic5;

implementation

{$R *.lfm}

{ Display details for actual circuit (latch) }

procedure ShowDetails(Circuit: Integer);

var
  I: Integer;

begin
  if Circuit <= 5 then
    fLogic5.stDetails.Caption := Details_SRLatch + LineEnding + LineEnding
  else
    fLogic5.stDetails.Caption := Details_DLatch + LineEnding + LineEnding;
  for I := 0 to 2 do
    fLogic5.stDetails.Caption := fLogic5.stDetails.Caption + Details_Circuits[Circuit, I] + LineEnding;
  if Circuit <= 2 then begin
    fLogic5.stDetails.Caption := fLogic5.stDetails.Caption + Details_Output1b + LineEnding;
    fLogic5.stDetails.Caption := fLogic5.stDetails.Caption + Details_Output2b;
  end
  else begin
    fLogic5.stDetails.Caption := fLogic5.stDetails.Caption + Details_Output1 + LineEnding;
    fLogic5.stDetails.Caption := fLogic5.stDetails.Caption + Details_Output2;
  end;
end;

{**********}
{ TfLogic5 }
{**********}

{ Application start: Initialisation }

procedure TfLogic5.FormCreate(Sender: TObject);

begin
  iCircuit := 0;                                                               // default circuit: active low SR NAND gates latch
end;

{ Menu item "File > Exit": Exit application }

procedure TfLogic5.mFileExitClick(Sender: TObject);

begin
  Close;
end;

{ Menu item "Help > About": Display application about }

procedure TfLogic5.mHelpAboutClick(Sender: TObject);

var
  S: string;

begin
  S := 'Electronics simulation.' + LineEnding;
  S += 'Common logic gates based circuits: Latches.' + LineEnding + LineEnding;
  S += 'Version 1.0, © allu, June-July 2020.';
  MessageDlg('About "LogicGates5"', S, mtInformation, [mbOK], 0);
end;

{ Button "Show": Open simulation window for actual latch }

procedure TfLogic5.btShowClick(Sender: TObject);

begin
  case iCircuit of
    0: fSR.ShowModal;
    1: fRS.ShowModal;
    2: fSR2.ShowModal;
    3: fSRE.ShowModal;
    4: fRSE.ShowModal;
    6: fD1.ShowModal;
    7: fD2.ShowModal;
    8: fD1b.ShowModal;
  end;
end;

{ Latch selection (one of the eight radiobutton checked): Display circuit details }

procedure TfLogic5.rbCircuit0Change(Sender: TObject);

begin
  if rbCircuit0.Checked then begin
    iCircuit := 0;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic5.rbCircuit1Change(Sender: TObject);

begin
  if rbCircuit1.Checked then begin
    iCircuit := 1;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic5.rbCircuit2Change(Sender: TObject);

begin
  if rbCircuit2.Checked then begin
    iCircuit := 2;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic5.rbCircuit3Change(Sender: TObject);

begin
  if rbCircuit3.Checked then begin
    iCircuit := 3;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic5.rbCircuit4Change(Sender: TObject);

begin
  if rbCircuit4.Checked then begin
    iCircuit := 4;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic5.rbCircuit6Change(Sender: TObject);

begin
  if rbCircuit6.Checked then begin
    iCircuit := 6;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic5.rbCircuit7Change(Sender: TObject);

begin
  if rbCircuit7.Checked then begin
    iCircuit := 7;
    ShowDetails(iCircuit);
  end;
end;

procedure TfLogic5.rbCircuit8Change(Sender: TObject);

begin
  if rbCircuit8.Checked then begin
    iCircuit := 8;
    ShowDetails(iCircuit);
  end;
end;

end.

