{*********************************************}
{* SR latch unit for LogicGates5 application *}
{*********************************************}

unit sr;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, common;

type
  {******}
  { TfSR }
  {******}
  TfSR = class(TForm)
    stTitle: TStaticText;
    imGate1: TImage;
    imGate2: TImage;
    shSwitch1a, shSwitch1b, shSwitch2a, shSwitch2b: TShape;
    imSwitch1, imSwitch2: TImage;
    shLedIn1, shLedIn2, shLedOut1, shLedOut2: TShape;
    Shape1, Shape2, Shape3, Shape4, Shape5, Shape6, Shape7, Shape8, Shape9, Shape10: TShape;
    Shape11, Shape12, Shape13, Shape14, Shape15, Shape16, Shape17, Shape18, Shape19: TShape;
    Shape20, Shape21, Shape22, Shape23, Shape24, Shape25, Shape26, Shape27, Shape28: TShape;
    Shape29, Shape30, Shape31, Shape32, Shape37: TShape;
    StaticText1, StaticText2, StaticText3, StaticText4, StaticText5, StaticText6: TStaticText;
    StaticText7, StaticText8, StaticText9, StaticText10, StaticText11: TStaticText;
    Label1, Label2, Label3, Label4, Label5, Label6, Label7, Label8, Label10: TLabel;
    Label11, Label12, Label13, Label14, Label15, Label16, Label17, Label18, Label19: TLabel;
    Label20, Label21, Label22, Label23, Label24, Label25, Label26, Label27, Label28: TLabel;
    Label29, Label30, Label31, Label32, Label33, Label34, Label35, Label36, Label37: TLabel;
    Label38, Label39, Label40, Label41, Label42, Label43, Label44, Label45, Label46: TLabel;
    Memo1: TMemo;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitch1Click(Sender: TObject);
    procedure imSwitch2Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    Sbar, Rbar, Q, Qbar: Boolean;
    SwitchSet, SwitchReset: TSwitch;
    LedSet, LedReset, LedQ, LedQbar: TLed;
  end;

var
  fSR: TfSR;

implementation

{$R *.lfm}

{******}
{ TfSR }
{******}

{ Application start: Init the LED and Switch records (would be object creation in OOP) }

procedure TfSR.FormCreate(Sender: TObject);

begin
  LedSet.LedShape := shLedIn1; LedReset.LedShape := shLedIn2;                  // SET and RESET input LEDs
  LedQ.LedShape := shLedOut1;  LedQbar.LedShape := shLedOut2;                  // Q and Qbar output LEDs
  with SwitchSet do begin                                                      // SET switch
    SwitchShapeA := shSwitch1a; SwitchShapeB := shSwitch1b;
    SwitchShapeAOffPos := 100;                                                 // vertical position (Top property) of open switch
  end;
  with SwitchReset do begin                                                    // RESET switch
    SwitchShapeA := shSwitch2a; SwitchShapeB := shSwitch2b;
    SwitchShapeAOffPos := 341;                                                 // vertical position (Top property) of open switch
  end;
end;

{ Window show-up: Init the latch circuit (initial state in table) }

procedure TfSR.FormActivate(Sender: TObject);

begin
  SwitchOff(SwitchSet); SwitchOff(SwitchReset);
  LedOn(LedSet); LedOn(LedReset); LedOff(LedQ); LedOn(LedQbar);
  Sbar := True; Rbar := True;                                                  // logics of the 2 inputs (as Boolean)
  Q := False; Qbar := True;                                                    // logics of the 2 outputs (as Boolean)
end;

{ SET switch pushed: Set corr. input as applicable and update simulation accordingly }

procedure TfSR.imSwitch1Click(Sender: TObject);

var
  Forbidden: Boolean;

begin
  if SwitchSet.SwitchOn then begin
    // Switch SET was on
    SwitchOff(SwitchSet); LedOn(LedSet);                                       // switch off, LED on (active low!)
    Sbar := True;                                                              // Sbar = logic 1
  end
  else begin
    // Switch SET was off
    SwitchOn(SwitchSet); LedOff(LedSet);                                       // switch on, LED off (active low!)
    Sbar := False;                                                             // Sbar = logic 0
  end;
  Latch_SR_nand('set', Sbar, Rbar, Q, Qbar, LedQ, LedQbar, Forbidden);         // active low NAND gates SR latch simulation
  if Forbidden then
    // Forbidden state message
    MessageDlg('Forbidden state', 'The two inputs of an active low NAND gates SR latch must not be at logic 0 at the same time!', mtWarning, [mbOK], 0);
end;

{ RESET switch pushed: Set corr. input as applicable and update simulation accordingly }

procedure TfSR.imSwitch2Click(Sender: TObject);

var
  Forbidden: Boolean;

begin
  if SwitchReset.SwitchOn then begin
    // Switch RESET was on
    SwitchOff(SwitchReset); LedOn(LedReset);
    Rbar := True;                                                              // Rbar = logic 1
  end
  else begin
    // Switch RESET was off
    SwitchOn(SwitchReset); LedOff(LedReset);
    Rbar := False;                                                             // Sbar = logic 0
  end;
  Latch_SR_nand('reset', Sbar, Rbar, Q, Qbar, LedQ, LedQbar, Forbidden);       // active low NAND gates SR latch simulation
  if Forbidden then
    // Forbidden state message
    MessageDlg('Forbidden state', 'The two inputs of an active low NAND gates SR latch must not be at logic 0 at the same time!', mtWarning, [mbOK], 0);
end;

{Button "Close": Close the simulation window }

procedure TfSR.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

