{******************************************************}
{* Positive SR latch unit for LogicGates5 application *}
{******************************************************}

unit sr2;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, common;

type
  {*******}
  { TfSR2 }
  {*******}
  TfSR2 = class(TForm)
    stTitle: TStaticText;
    imGate1, imGate2, imGate3, imGate4: TImage;
    shSwitch1a, shSwitch1b, shSwitch2a, shSwitch2b: TShape;
    imSwitch1, imSwitch2: TImage;
    shLedIn1, shLedIn2, shLedIn1b, shLedIn2b, shLedOut1, shLedOut2: TShape;
    Shape1, Shape2, Shape3, Shape4, Shape5, Shape6, Shape7, Shape8, Shape9, Shape10: TShape;
    Shape11, Shape12, Shape13, Shape14, Shape15, Shape16, Shape17, Shape18, Shape19, Shape20: TShape;
    Shape21, Shape22, Shape23, Shape24, Shape25, Shape26, Shape27, Shape28, Shape29, Shape30: TShape;
    Shape31, Shape32, Shape33, Shape37, Shape38, Shape39, Shape40, Shape41, Shape42, Shape43, Shape44: TShape;
    Shape45, Shape46, Shape47, Shape48, Shape49: TShape;
    StaticText1, StaticText2, StaticText3, StaticText4, StaticText5, StaticText6, StaticText7: TStaticText;
    StaticText8, StaticText9, StaticText10, StaticText11, StaticText12, StaticText13: TStaticText;
    Label1, Label2, Label3, Label4, Label5, Label6, Label7, Label8, Label9, Label10: TLabel;
    Label11, Label12, Label13, Label14, Label15, Label16, Label17, Label18, Label19, Label20: TLabel;
    Label21, Label22, Label23, Label24, Label25, Label26, Label27, Label28, Label29, Label30: TLabel;
    Label31, Label32, Label33, Label34, Label35, Label36, Label37, Label38, Label39, Label40: TLabel;
    Label41, Label42, Label43, Label44, Label45, Label46, Label47: TLabel;
    Memo1: TMemo;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitch1Click(Sender: TObject);
    procedure imSwitch2Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    S, R, Q, Qbar: Boolean;
    SwitchSet, SwitchReset: TSwitch;
    LedSet, LedReset, LedSet2, LedReset2, LedQ, LedQbar: TLed;
  end;

var
  fSR2: TfSR2;

implementation

{$R *.lfm}

{*******}
{ TfSR2 }
{*******}

{ Application start: Init the LED and Switch records }

procedure TfSR2.FormCreate(Sender: TObject);

begin
  LedSet.LedShape := shLedIn1; LedReset.LedShape := shLedIn2;                  // S and R inout LEDs
  LedSet2.LedShape := shLedIn1b; LedReset2.LedShape := shLedIn2b;              // Sbar and Rbar input LEDs
  LedQ.LedShape := shLedOut1;  LedQbar.LedShape := shLedOut2;                  // Q and Qbar output LEDs
  with SwitchSet do begin                                                      // SET switch
    SwitchShapeA := shSwitch1a; SwitchShapeB := shSwitch1b;
    SwitchShapeAOffPos := 100;
  end;
  with SwitchReset do begin                                                    // RESET switch
    SwitchShapeA := shSwitch2a; SwitchShapeB := shSwitch2b;
    SwitchShapeAOffPos := 341;
  end;
end;

{ Window show-up: Init latch circuit (initial state in table) }

procedure TfSR2.FormActivate(Sender: TObject);

begin
  SwitchOff(SwitchSet); SwitchOff(SwitchReset);
  LedOff(LedSet); LedOff(LedReset); LedOn(LedSet2); LedOn(LedReset2);
  LedOff(LedQ); LedOn(LedQbar);
  S := False; R := False;                                                      // logics of S and R inputs (as Boolean)
  Q := False; Qbar := True;                                                    // logics of Q and Qbar outputs (as Boolean)
end;

{ SET switch pushed: Set corr. input as applicable and update simulation accordingly }

procedure TfSR2.imSwitch1Click(Sender: TObject);

var
  Forbidden: Boolean;

begin
  if SwitchSet.SwitchOn then begin
    // Switch SET switch was on
    SwitchOff(SwitchSet);                                                      // SET switch off
    LedOff(LedSet); LedOn(LedSet2);                                            // S LED off => Sbar LED on
    S := False;                                                                // S = logic 0
  end
  else begin
    // Switch SET switch was off
    SwitchOn(SwitchSet);                                                       // SET switch on
    LedOn(LedSet); LedOff(LedSet2);                                            // S LED on => Sbar LED off
    S := True;                                                                 // S = logic 1
  end;
  Latch_SR_nand2('set', S, R, Q, Qbar, LedQ, LedQbar, Forbidden);              // active high NAND gates SR latch simulation
  if Forbidden then
    // Forbidden state message
    MessageDlg('Forbidden state', 'The two inputs of an active high NAND gates SR latch must not be at logic 1 at the same time!', mtWarning, [mbOK], 0);
end;

{ RESET switch pushed: Set corr. input as applicable and update simulation accordingly }

procedure TfSR2.imSwitch2Click(Sender: TObject);

var
  Forbidden: Boolean;

begin
  if SwitchReset.SwitchOn then begin
    // Switch RESET was on
    SwitchOff(SwitchReset);
    LedOff(LedReset); LedOn(LedReset2);
    R := False;                                                                // R = logic 0
  end
  else begin
    // Switch RESET was off
    SwitchOn(SwitchReset);
    LedOn(LedReset); LedOff(LedReset2);
    R := True;                                                                 // R = logic 1
  end;
  Latch_SR_nand2('reset', S, R, Q, Qbar, LedQ, LedQbar, Forbidden);            // active high NAND gates SR latch simulation
  if Forbidden then
    // Forbidden state message
    MessageDlg('Forbidden state', 'The two inputs of an active high NAND gates SR latch must not be at logic 1 at the same time!', mtWarning, [mbOK], 0);
end;

{ Button "Close": Close simulation window }

procedure TfSR2.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

